% close all;


% Time
t_start = 0;
t_step = 0.01;
t_stop = 10;
time = (t_start:t_step:t_stop)';



% Wall
global x_wall eta rho0 beta;
x_wall = 0.8;
rho0 = 0.1;
eta = 1;
beta = 0.99;
error = 0;


% References
r = [0, (1+error)*x_wall, 1, 0, 0, -1.57]';
Fd = [0, -10, 0, 0, 0, 0]';
F_meter_max = -30;

if select_position == 0
    r = r*ones(1,length(time));
    rd = zeros(6,length(time));
    rdd = zeros(6,length(time));
else
    amp = 0.2;
    freq = 0.9;
    r = r*ones(1,length(time)) + [amp*cos(freq*time), zeros(length(time),1), amp*sin(freq*time), zeros(length(time),1), zeros(length(time),1), zeros(length(time),1)]';
    rd = [-amp*freq*sin(freq*time), zeros(length(time),1), amp*freq*cos(freq*time), zeros(length(time),1), zeros(length(time),1), zeros(length(time),1)]';
    rdd = [-amp*freq^2*cos(freq*time), zeros(length(time),1), -amp*freq^2*sin(freq*time), zeros(length(time),1), zeros(length(time),1), zeros(length(time),1)]';
end

if select_force == 0
    Fd = Fd*ones(1,length(time)) + [zeros(6,(t_stop/(2*t_step))), Fd*ones(1,length(time)-(t_stop/(2*t_step)))];
elseif select_force == 1
    Fd = Fd*ones(1,length(time)) + [zeros(length(time),1), -time, zeros(length(time),1), zeros(length(time),1), zeros(length(time),1), zeros(length(time),1)]';
else
    Fd = Fd*ones(1,length(time)) + [zeros(length(time),1), 2*sin(2*time), zeros(length(time),1), zeros(length(time),1), zeros(length(time),1), zeros(length(time),1)]';
end



% Links
n_links = 6;
syms q1 q2 q3 q4 q5 q6 ...
q1d q2d q3d q4d q5d q6d real;
q = [q1, q2, q3, q4, q5, q6]';
qd = [q1d, q2d, q3d, q4d, q5d, q6d]';




% Parameters
global g m lc I1 I2 I3 I4 I5 I6;
syms pi real;

if symbolic
    syms g ...
    lc1 lc2 lc3 lc4 lc5 lc6 ...         % Link CG position
    m1 m2 m3 m4 m5 m6 ...               % Link masses
    I1xx I1yy I1zz I1xy I1xz I1yz ...   % Link inertia matrices
    I2xx I2yy I2zz I2xy I2xz I2yz ...
    I3xx I3yy I3zz I3xy I3xz I3yz ...
    I4xx I4yy I4zz ...
    I5xx I5yy I5zz ...
    I6xx I6yy I6zz real;
else
    g = 9.81;
    [m1, m2, m3, m4, m5, m6] = deal(10, 10, 10, 1, 1, 1);
    [lc1, lc2, lc3, lc4, lc5, lc6] = deal(0.1, 0.5, 0.3, 0.1, 0.025, 0.075);
    [I1xx, I1yy, I1zz] = deal(1, 1, 1);
    [I2xx, I2yy, I2zz] = deal(1, 1, 1);
    [I3xx, I3yy, I3zz] = deal(1, 1, 1);
    [I4xx, I4yy, I4zz] = deal(0.1, 0.1, 0.1);
    [I5xx, I5yy, I5zz] = deal(0.1, 0.1, 0.1);
    [I6xx, I6yy, I6zz] = deal(0.1, 0.1, 0.1);
end

m = [m1, m2, m3, m4, m5, m6]';
lc = [lc1, lc2, lc3, lc4, lc5, lc6]';
I1 = diag([I1xx, I1yy, I1zz]);
I2 = diag([I2xx, I2yy, I2zz]);
I3 = diag([I3xx, I3yy, I3zz]);
I4 = diag([I4xx, I4yy, I4zz]);
I5 = diag([I5xx, I5yy, I5zz]);
I6 = diag([I6xx, I6yy, I6zz]);


% DH parameters
global d a;

if symbolic
    syms a1 a2 a3 d1 d4 d6 real;
else
    a1 = 0.2;
    a2 = 1;
    a3 = 0.1;
    d1 = 0.2;
    d4 = 0.5;
    d6 = 0.1;
end

theta = [pi/2; q];
theta(4) = theta(4) + pi/2;
theta(7) = theta(7) + pi;
d = [0, -d1, 0, 0, -d4, 0, -d6]';
a = [0, a1, a2, a3, 0, 0, 0]';
alpha = (pi/2)*[2, 1, 0, -1, 1, -1, 2]';

dh_table = [theta, d, a, alpha];


% Damped Jacobian
J_damp = 0.0001;






% Initial values
if ~symbolic
    q0 = inv_kin(r(1:3,1)+[0, -0.2, 0]', R_zyx([0, 1, 0]'), 1,0,1);
    qd0 = [0, 0, 0, 0, 0, 0]';
end



% Controller
if hybrid_controller
    S_p = diag([1 0 1 1 1 1]);
    S_f = diag([0 1 0 0 0 0]);
else
    S_p = eye(6);
    S_f = zeros(6);
end

K_pd = diag([40, 40, 40, 20, 20, 20]);
K_pp = (K_pd.^2)/4;             % Critically damped
K_fp = diag([10, 10, 10, 1, 1, 1]);
K_fd = diag([40, 40, 40, 10, 10, 10]);



