function [ q ] = inv_kin( o, R, elbow_right, elbow_down, wrist_up )

    global a d;
    a1 = a(2);
    a2 = a(3);
    a3 = a(4);
    d1 = -d(2);
    d4 = -d(5);
    d6 = -d(7);
    q = zeros(6, 1);
    
    
    % Inverse position kinematics
    oc = o - d6*R(1:3,3);
    
    % q1
    if elbow_right
        q(1) = atan2(oc(1), oc(2));
    else
        q(1) = atan2(-oc(1), -oc(2));
    end

    op = d1*[0 0 1]' + a1*[sin(q(1)), cos(q(1)), 0]';
    
    
    
    
    % q1, q3
    rr = (oc(1)-op(1))^2 + (oc(2)-op(2))^2;
    ss = (oc(3)-op(3))^2;
    kk = a3^2 + d4^2;
    r = sqrt(rr);
    s = sqrt(ss);
    k = sqrt(kk);
    
    D = (rr + ss - a2^2 - kk)/(2*a2*k);
    alpha = atan(d4/a3);
    
    if elbow_down
        theta3 = atan2(sqrt(1-D^2), D);
    else
        theta3 = atan2(-sqrt(1-D^2), D);
    end
    theta2 = atan2(s,r) - atan2(k*sin(theta3), a2+k*cos(theta3));
    
    if ~elbow_right
        theta2 = pi - theta2;
    end
    
    q(2) = -theta2;
    q(3) = alpha - pi/2 - theta3;
    
    
    
    % Inverse orientation kinematics
    r13 = -R(2,3)*sin(q(2) + q(3))*cos(q(1)) - R(1,3)*sin(q(2) + q(3))*sin(q(1)) - R(3,3)*cos(q(2) + q(3));
    r23 = -R(2,3)*sin(q(1)) + R(1,3)*cos(q(1));
    r33 = -R(3,3)*sin(q(2) + q(3)) + R(2,3)*cos(q(2) + q(3))*cos(q(1)) + R(1,3)*cos(q(2) + q(3))*sin(q(1));
    r31 = R(3,1)*sin(q(2) + q(3)) - R(1,1)*cos(q(2) + q(3))*sin(q(1)) - R(2,1)*cos(q(2) + q(3))*cos(q(1));
    r32 = R(3,2)*sin(q(2) + q(3)) - R(1,2)*cos(q(2) + q(3))*sin(q(1)) - R(2,2)*cos(q(2) + q(3))*cos(q(1));

    if wrist_up
        q(5) = atan2(sqrt(1-r33^2), r33);
        q(4) = atan2(r23, r13);
        q(6) = atan2(-r32, -r31);
    else
        q(5) = atan2(-sqrt(1-r33^2), r33);
        q(4) = atan2(-r23, -r13);
        q(6) = atan2(r32, r31);
    end
    
end
